/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.model;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import com.redhat.rhevm.api.common.util.StatusUtils;
import com.redhat.rhevm.api.model.Storage;
import com.redhat.rhevm.api.model.StorageDomain;
import com.redhat.rhevm.api.model.StorageDomainType;
import com.redhat.rhevm.api.model.StorageType;
import com.redhat.rhevm.api.powershell.enums.PowerShellStorageDomainStatus;
import com.redhat.rhevm.api.powershell.enums.PowerShellStorageType;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;

public class PowerShellStorageDomain extends StorageDomain {

    private static final String STORAGE_DOMAIN_TYPE = "RhevmCmd.CLIStorageDomain";

    private String vgId;

    static boolean isStorageDomain(PowerShellParser.Entity entity) {
        return STORAGE_DOMAIN_TYPE.equals(entity.getType());
    }

    public String getVgId() {
        return vgId;
    }
    public void setVgId(String vgId) {
        this.vgId = vgId;
    }

    public static List<PowerShellStorageDomain> parse(PowerShellParser parser, String output) {
        List<PowerShellStorageDomain> ret = new ArrayList<PowerShellStorageDomain>();

        for (PowerShellParser.Entity entity : parser.parse(output)) {
            if (isStorageDomain(entity)) {
                ret.add(parseEntity(entity));
            }
        }

        return ret;
    }

    public static PowerShellStorageDomain parseEntity(PowerShellParser.Entity entity) {
        PowerShellStorageDomain storageDomain = new PowerShellStorageDomain();

        storageDomain.setId(entity.get("storagedomainid"));
        storageDomain.setName(entity.get("name"));
        storageDomain.setVgId(entity.get("vgid"));

        String domainType = entity.get("domaintype").toUpperCase();
        storageDomain.setMaster(domainType.endsWith(" (MASTER)"));
        if (storageDomain.isMaster()) {
            domainType = domainType.split(" ")[0];
        }
        storageDomain.setType(StorageDomainType.valueOf(domainType).value());

        PowerShellStorageDomainStatus status =
            entity.get("status", PowerShellStorageDomainStatus.class);
        if (status != null) {
            storageDomain.setStatus(StatusUtils.create(status.map()));
        }

        Storage storage = new Storage();

        StorageType storageType = entity.get("type", PowerShellStorageType.class).map();

        storage.setType(storageType.value());

        switch (storageType) {
        case NFS:
            String[] parts = entity.get("nfspath").split(":");
            storage.setAddress(parts[0]);
            storage.setPath(parts[1]);
            break;
        case ISCSI:
            break;
        case FCP:
        default:
            assert false : storage.getType();
            break;
        }

        storageDomain.setStorage(storage);

        parseUnsignedInt(entity, storageDomain, "availabledisksize", "Available");
        parseUnsignedInt(entity, storageDomain, "useddisksize", "Used");
        parseUnsignedInt(entity, storageDomain, "committeddisksize", "Committed");

        return storageDomain;
    }

    private static void parseUnsignedInt(PowerShellParser.Entity entity, PowerShellStorageDomain storageDomain, String property, String field) {
        Integer value = entity.get(property, Integer.class);
        if (value != null) {
            try {
                Method m = storageDomain.getClass().getMethod("set" + field, Long.class);
                m.invoke(storageDomain, Long.valueOf(value) * 1024 * 1024 * 1024);
            } catch (Exception e) {
                // simple setter shouldn't fail
            }
        }
    }
}
